@@ -615,7 +615,7 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
                 except:
                     pass
             if new_data is None or (type(data_to_store) == np.unicode_ \
-                    and len(data_to_store) == len(new_data)) \
+                    and len(data_to_store) != len(new_data)) \
                     or (isinstance(data_to_store, np.ndarray) \
                     and new_data.shape[-1] != data_to_store.shape[-1] \
                     * (data_to_store.dtype.itemsize//4)):
@@ -680,6 +680,9 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
         # converting that field to a HDF5 Reference types, it will just
         # be written as a Group instead (just have to see if ", 'O'" is
         # in str(data_to_store.dtype).
+        #
+        # A flag, wrote_as_struct, is set depending on which path is
+        # taken, which is then passed onto write_metadata.
         
         if data_to_store.dtype.fields is not None \
                 and h5py.check_dtype(ref=data_to_store.dtype) \
@@ -690,6 +693,7 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
                 or not all(data_to_store.shape) \
                 or not all([all(data_to_store[n].shape) \
                 for n in data_to_store.dtype.names])):
+            wrote_as_struct = True
             # Grab the list of fields that don't have a null character
             # or a / in them since those can't be written.
             field_names = [n for n in data_to_store.dtype.names
@@ -772,6 +776,7 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
                                 - set(['H5PATH'])):
                             del_attribute(grp2[field], attribute)
         else:
+            wrote_as_struct = False
             # If it has fields and it isn't a Reference type, none of
             # them can contain a / character.
             if data_to_store.dtype.fields is not None \
@@ -848,9 +853,14 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
 
         # Write the metadata using the inherited function (good enough).
         self.write_metadata(f, grp, name, data, type_string,
-                            options)
+                            options, wrote_as_struct=wrote_as_struct)
+
+    def write_metadata(self, f, grp, name, data, type_string, options,
+                       wrote_as_struct=False):
+        # wote_as_struct is used to pass whether data was written like a
+        # matlab struct or not. If yes, then the field names must be put
+        # in the metadata.
 
-    def write_metadata(self, f, grp, name, data, type_string, options):
         # First, call the inherited version to do most of the work.
 
         TypeMarshaller.write_metadata(self, f, grp, name, data,
@@ -889,19 +899,12 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
             set_attribute_string(grp[name], 'Python.numpy.Container',
                                  container)
 
-        # If its dtype has fields and we would have written it as a
-        # Group (option is set, one of the field dtypes is object, or if
-        # data or one of its fields are empty), then we set the
+        # If it was written like a matlab struct, then we set the
         # 'Python.Fields' and 'MATLAB_fields' Attributes to the field
         # names if we are storing python metadata or doing matlab
         # compatibility and we are storing a structured ndarray as a
         # structure.
-        if data.dtype.fields is not None \
-                and (options.structured_numpy_ndarray_as_struct \
-                or "'O'" in str(data.dtype) \
-                or not all(data.shape) \
-                or not all([all(data[n].shape) \
-                for n in data.dtype.names])):
+        if wrote_as_struct:
             # Grab the list of fields. They need to be converted to
             # unicode in Python 2.x.
             if sys.hexversion >= 0x03000000:
@@ -911,9 +914,7 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
                                for c in list(data.dtype.names)]
 
             # Write or delete 'Python.Fields' as appropriate.
-            if options.store_python_metadata \
-                    and data.dtype.fields is not None \
-                    and options.structured_numpy_ndarray_as_struct:
+            if options.store_python_metadata:
                 set_attribute_string_array(grp[name],
                                            'Python.Fields',
                                            field_names)
@@ -1008,25 +1009,55 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
 
         # Get the different attributes this marshaller uses.
 
-        type_string = get_attribute_string(grp[name], 'Python.Type')
-        underlying_type = get_attribute_string(grp[name], \
-            'Python.numpy.UnderlyingType')
-        shape = get_attribute(grp[name], 'Python.Shape')
-        container = get_attribute_string(grp[name], \
-            'Python.numpy.Container')
-        python_empty = get_attribute(grp[name], 'Python.Empty')
-        python_fields = get_attribute_string_array(grp[name], \
-            'Python.Fields')
+        if sys.hexversion >= 0x03000000:
+            defaultfactory = type(None)
+        else:
+            defaultfactory = lambda : None
+        attributes = collections.defaultdict(defaultfactory,
+                                             grp[name].attrs.items())
 
-        matlab_class = get_attribute_string(grp[name], 'MATLAB_class')
-        matlab_empty = get_attribute(grp[name], 'MATLAB_empty')
+        str_attrs = dict()
+        for attr_name in ('Python.Type',
+                          'Python.numpy.UnderlyingType',
+                          'Python.numpy.Container',
+                          'MATLAB_class'):
+            value = attributes[attr_name]
+            if value is None:
+                str_attrs[attr_name] = value
+            elif (sys.hexversion >= 0x03000000 \
+                    and isinstance(value, str)) \
+                    or (sys.hexversion < 0x03000000 \
+                    and isinstance(value, unicode)):
+                str_attrs[attr_name] = value
+            elif isinstance(value, bytes):
+                str_attrs[attr_name] = value.decode()
+            elif isinstance(value, np.unicode_):
+                str_attrs[attr_name] = str(value)
+            elif isinstance(value, np.bytes_):
+                str_attrs[attr_name] = value.decode()
+            else:
+                str_attrs[attr_name] = None
+
+        type_string = str_attrs['Python.Type']
+        underlying_type = str_attrs['Python.numpy.UnderlyingType']
+        container = str_attrs['Python.numpy.Container']
+        matlab_class = str_attrs['MATLAB_class']
+
+        shape = attributes['Python.Shape']
+        python_empty = attributes['Python.Empty']
+        matlab_empty = attributes['MATLAB_empty']
+
+        python_fields = attributes['Python.Fields']
+        if python_fields is not None:
+            python_fields = [convert_to_str(x)
+                             for x in python_fields]
 
         # If we are using h5py version >= 2.3, we can actually read the
         # MATLAB_fields Attribute if it is present.
         matlab_fields = None
         if distutils.version.LooseVersion(_H5PY_VERSION) \
                 >= distutils.version.LooseVersion('2.3'):
-            matlab_fields = get_attribute(grp[name], 'MATLAB_fields')
+            matlab_fields = attributes['MATLAB_fields']
 
         # If it is a Dataset, it can simply be read and then acted upon
         # (if it is an HDF5 Reference array, it will need to be read
@@ -1692,8 +1723,10 @@ class PythonListMarshaller(NumpyScalarArrayMarshaller):
         # to be grabbed now as the parent function will have a modified
         # form of data to guess from if not given the right one
         # explicitly.
+        out = np.zeros(dtype='object', shape=(len(data), ))
+        out[:] = data
         NumpyScalarArrayMarshaller.write(self, f, grp, name,
-                                         np.object_(data),
+                                         out,
                                          self.get_type_string(data,
                                          type_string), options)
 
