@@ -28,11 +28,11 @@ This is the hdf5storage package, a Python package to read and write
 python data types to HDF5 (Heirarchal Data Format) files beyond just
 Numpy types.
 
-Version 0.1.14
+Version 0.1.15
 
 """
 
-__version__ = "0.1.14"
+__version__ = "0.1.15"
 
 import sys
 import os
@@ -441,7 +441,7 @@ class Options(object):
 
     @property
     def convert_numpy_bytes_to_utf16(self):
-        """ Whether or not to convert numpy.bytes_ to UTF-16.
+        """ Whether or not to convert numpy.bytes\_ to UTF-16.
 
         bool
 
@@ -455,7 +455,7 @@ class Options(object):
 
         See Also
         --------
-        numpy.bytes_
+        numpy.bytes\_
         convert_numpy_str_to_utf16
 
         """
@@ -472,7 +472,7 @@ class Options(object):
 
     @property
     def convert_numpy_str_to_utf16(self):
-        """ Whether or not to convert numpy.str_ to UTF-16.
+        """ Whether or not to convert numpy.str\_ to UTF-16.
 
         bool
 
@@ -491,7 +491,7 @@ class Options(object):
 
         See Also
         --------
-        numpy.bytes_
+        numpy.bytes\_
         convert_numpy_str_to_utf16
 
         """
