@@ -30,7 +30,6 @@
 
 import sys
 import copy
-import string
 import random
 
 import numpy as np
@@ -60,17 +59,31 @@ def next_unused_name_in_group(grp, length):
         `grp`.
 
     """
-    ltrs = string.ascii_letters + string.digits
-    existing_names = set(grp.keys())
-    while True:
-        name = ''.join([random.choice(ltrs) for i in range(0, length)])
-        if name not in existing_names:
-            return name
+    # While
+    #
+    # ltrs = string.ascii_letters + string.digits
+    # name = ''.join([random.choice(ltrs) for i in range(length)])
+    #
+    # seems intuitive, its performance is abysmal compared to
+    #
+    # '%0{0}x'.format(length) % random.getrandbits(length * 4)
+    #
+    # The difference is a factor of 20. Idea from
+    #
+    # https://stackoverflow.com/questions/2782229/most-lightweight-way-
+    #   to-create-a-random-string-and-a-random-hexadecimal-number/
+    #   35161595#35161595
+    fmt = '%0{0}x'.format(length)
+    name = fmt % random.getrandbits(length * 4)
+    while name in grp:
+        name = fmt % random.getrandbits(length * 4)
+    return name
+
 
 def convert_numpy_str_to_uint16(data):
-    """ Converts a numpy.str_ to UTF-16 encoding in numpy.uint16 form.
+    """ Converts a numpy.unicode\_ to UTF-16 in numpy.uint16 form.
 
-    Convert a ``numpy.str`` or an array of them (they are UTF-32
+    Convert a ``numpy.unicode_`` or an array of them (they are UTF-32
     strings) to UTF-16 in the equivalent array of ``numpy.uint16``. The
     conversion will throw an exception if any characters cannot be
     converted to UTF-16. Strings are expanded along rows (across columns)
@@ -80,12 +93,12 @@ def convert_numpy_str_to_uint16(data):
 
     Parameters
     ----------
-    data : numpy.str_ or numpy.ndarray of numpy.str_
+    data : numpy.unicode\_ or numpy.ndarray of numpy.unicode\_
         The string or array of them to convert.
 
     Returns
     -------
-    numpy.ndarray of numpy.uint16
+    array : numpy.ndarray of numpy.uint16
         The result of the conversion.
 
     Raises
@@ -96,56 +109,34 @@ def convert_numpy_str_to_uint16(data):
     See Also
     --------
     convert_numpy_str_to_uint32
-    decode_to_numpy_str
+    convert_to_numpy_str
 
     """
     # An empty string should be an empty uint16
     if data.nbytes == 0:
         return np.uint16([])
 
-    # If it is just a string instead of an array of them, then the
-    # string can simply be converted and returned as a 1d array pretty
-    # easily using ndarray's buffer option. The byte order mark, 2
-    # bytes, needs to be removed.
-    if not isinstance(data, np.ndarray):
-        s = data.encode(encoding='UTF-16', errors='strict')
-        return np.ndarray(shape=((len(s)-2)//2,), dtype='uint16',
-                          buffer=s[2:])
+    # We need to use the UTF-16 codec for our endianness. Using the
+    # right one means we don't have to worry about removing the BOM.
+    if sys.byteorder == 'little':
+        codec = 'UTF-16LE'
+    else:
+        codec = 'UTF-16BE'
 
-    # It is an array of strings. Each string in the array needs to be
-    # converted. An object array is needed to hold all the converted
-    # forms, as opposed to just constructing the final uint16 array,
-    # because the converted forms could end up greatly differing lengths
-    # depending on how many characters turn into doublets. The sizes of
-    # each one need to be grabbed along the way to be able to construct
-    # the final array. The easiest way to convert each string is to use
-    # recursion.
-    converted_strings = np.ndarray(shape=data.shape, dtype='object')
-    sizes = np.zeros(shape=data.shape, dtype='int64')
+    # numpy.char.encode can do the conversion element wise. Then, we
+    # just have convert to uin16 with the appropriate dimensions. The
+    # dimensions are gotten from the shape of the converted data with
+    # the number of column increased by the number of words (pair of
+    # bytes) in the strings.
+    cdata = np.char.encode(np.atleast_1d(data), codec)
+    shape = list(cdata.shape)
+    shape[-1] *= (cdata.dtype.itemsize // 2)
+    return np.ndarray(shape=shape, dtype='uint16',
+                      buffer=cdata.tostring())
 
-    for index, x in np.ndenumerate(data):
-        converted_strings[index] = convert_numpy_str_to_uint16(x)
-        sizes[index] = np.prod(converted_strings[index].shape)
-
-    # The shape of the new array is simply the shape of the old one with
-    # the number of columns increased multiplicatively by the size of
-    # the largest UTF-16 string so that everything will fit.
-    length = np.max(sizes)
-    shape = list(data.shape)
-    shape[-1] *= length
-    new_data = np.zeros(shape=tuple(shape), dtype='uint16')
-
-    # Copy each string into new_data using clever indexing (using the
-    # first part of index returns a 1d subarray that can be
-    # addressed). Then, the conversion is done.
-    for index, x in np.ndenumerate(converted_strings):
-        new_data[index[:-1]][ \
-            (length*index[-1]):(length*index[-1]+sizes[index])] = x
-
-    return new_data
 
 def convert_numpy_str_to_uint32(data):
-    """ Converts a numpy.str_ to its numpy.uint32 representation.
+    """ Converts a numpy.str\_ to its numpy.uint32 representation.
 
     Convert a ``numpy.str`` or an array of them (they are UTF-32
     strings) into the equivalent array of ``numpy.uint32`` that is byte
@@ -155,7 +146,7 @@ def convert_numpy_str_to_uint32(data):
 
     Parameters
     ----------
-    data : numpy.str_ or numpy.ndarray of numpy.str_
+    data : numpy.str\_ or numpy.ndarray of numpy.str\_
         The string or array of them to convert.
 
     Returns
@@ -241,7 +232,7 @@ def convert_to_str(data):
 
 
 def convert_to_numpy_str(data, length=None):
-    """ Decodes data to Numpy unicode string (str_).
+    """ Decodes data to Numpy unicode string (str\_).
 
     Decodes `data` to Numpy unicode string (UTF-32), which is
     ``numpy.str_``, or an array of them. If it can't be decoded, it is
@@ -275,7 +266,7 @@ def convert_to_numpy_str(data, length=None):
 
     Returns
     -------
-    numpy.str_ or numpy.ndarray of numpy.str_ or data
+    numpy.str\_ or numpy.ndarray of numpy.str\_ or data
         If `data` can be decoded into a ``numpy.str_`` or a
         ``numpy.ndarray`` of them, the decoded version is returned.
         Otherwise, `data` is returned unchanged.
@@ -284,7 +275,7 @@ def convert_to_numpy_str(data, length=None):
     --------
     convert_to_str
     convert_to_numpy_bytes
-    numpy.str_
+    numpy.str\_
 
     """
     # The method of conversion depends on its type.
@@ -380,7 +371,7 @@ def convert_to_numpy_str(data, length=None):
 
 
 def convert_to_numpy_bytes(data, length=None):
-    """ Decodes data to Numpy UTF-8 econded string (bytes_).
+    """ Decodes data to Numpy UTF-8 econded string (bytes\_).
 
     Decodes `data` to a Numpy UTF-8 encoded string, which is
     ``numpy.bytes_``, or an array of them in which case it will be ASCII
@@ -412,7 +403,7 @@ def convert_to_numpy_bytes(data, length=None):
 
     Returns
     -------
-    numpy.bytes_ or numpy.ndarray of numpy.bytes_ or data
+    numpy.bytes\_ or numpy.ndarray of numpy.bytes\_ or data
         If `data` can be decoded into a ``numpy.bytes_`` or a
         ``numpy.ndarray`` of them, the decoded version is returned.
         Otherwise, `data` is returned unchanged.
@@ -421,7 +412,7 @@ def convert_to_numpy_bytes(data, length=None):
     --------
     convert_to_str
     convert_to_numpy_str
-    numpy.bytes_
+    numpy.bytes\_
 
     """
     # The method of conversion depends on its type.
@@ -670,10 +661,10 @@ def get_attribute(target, name):
     isn't.
 
     """
-    if name not in target.attrs:
-        return None
-    else:
+    try:
         return target.attrs[name]
+    except:
+        return None
 
 
 def get_attribute_string(target, name):
