@@ -31,6 +31,7 @@ import warnings
 import numpy as np
 import numpy.testing as npt
 
+
 def assert_equal(a, b):
     # Compares a and b for equality. If they are dictionaries, they must
     # have the same set of keys, after which they values must all be
@@ -100,7 +101,10 @@ def assert_equal_none_format(a, b):
         for k in b:
             assert_equal_none_format(a[k][0], b[k])
     elif type(b) in (list, tuple, set, frozenset, collections.deque):
-        assert_equal_none_format(a, np.object_(list(b)))
+        b_conv = np.zeros(dtype='object', shape=(len(b), ))
+        for i, v in enumerate(b):
+            b_conv[i] = v
+        assert_equal_none_format(a, b_conv)
     elif not isinstance(b, (np.generic, np.ndarray)):
         if b is None:
             # It should be np.float64([])
@@ -144,7 +148,15 @@ def assert_equal_none_format(a, b):
                     assert a.shape == b.shape
                     npt.assert_equal(a, b)
             else:
-                assert a.dtype == b.dtype
+                # If the original is structued, it is possible that the
+                # fields got out of order, in which case the dtype won't
+                # quite match. It will need to be checked just to make
+                # sure all pieces are there. Otherwise, the dtypes can
+                # be directly compared.
+                if b.dtype.fields is None:
+                    assert a.dtype == b.dtype
+                else:
+                    assert dict(a.dtype.fields) == dict(b.dtype.fields)
                 # Now, if b.shape is just all ones, then a.shape will
                 # just be (1,). Otherwise, we need to compare the shapes
                 # directly. Also, dimensions need to be squeezed before
@@ -192,7 +204,10 @@ def assert_equal_matlab_format(a, b):
         for k in b:
             assert_equal_matlab_format(a[k][0], b[k])
     elif type(b) in (list, tuple, set, frozenset, collections.deque):
-        assert_equal_matlab_format(a, np.object_(list(b)))
+        b_conv = np.zeros(dtype='object', shape=(len(b), ))
+        for i, v in enumerate(b):
+            b_conv[i] = v
+        assert_equal_matlab_format(a, b_conv)
     elif not isinstance(b, (np.generic, np.ndarray)):
         if b is None:
             # It should be np.zeros(shape=(0, 1), dtype='float64'))
